@@ -24,8 +24,6 @@ module CgForeignCall (
   emitOpenNursery,
  ) where
 
-#include "HsVersions.h"
-
 import StgSyn
 import CgProf
 import CgBindery
@@ -43,6 +41,8 @@ import ClosureInfo
 import Constants
 import StaticFlags
 import Outputable
+import FastString
+import BasicTypes
 
 import Control.Monad
 
@@ -50,7 +50,7 @@ import Control.Monad
 -- Code generation for Foreign Calls
 
 cgForeignCall
-	:: CmmHintFormals	-- where to put the results
+	:: CmmFormals	-- where to put the results
 	-> ForeignCall		-- the op
 	-> [StgArg]		-- arguments
 	-> StgLiveVars	-- live vars, in case we need to save them
@@ -64,15 +64,16 @@ cgForeignCall results fcall stg_args live
 	  	    | (stg_arg, (rep,expr)) <- stg_args `zip` reps_n_amodes, 
 	               nonVoidArg rep]
 
-	arg_hints = zip arg_exprs (map (typeHint.stgArgType) stg_args)
+	arg_hints = zipWith CmmKinded
+                      arg_exprs (map (typeHint.stgArgType) stg_args)
   -- in
   emitForeignCall results fcall arg_hints live
 
 
 emitForeignCall
-	:: CmmHintFormals	-- where to put the results
+	:: CmmFormals	-- where to put the results
 	-> ForeignCall		-- the op
-	-> [(CmmExpr,MachHint)] -- arguments
+	-> [CmmKinded CmmExpr] -- arguments
 	-> StgLiveVars	-- live vars, in case we need to save them
 	-> Code
 
@@ -85,15 +86,15 @@ emitForeignCall results (CCall (CCallSpec target cconv safety)) args live
       (call_args, cmm_target)
 	= case target of
 	   StaticTarget lbl -> (args, CmmLit (CmmLabel 
-					(mkForeignLabel lbl call_size False)))
-	   DynamicTarget    ->  case args of (fn,_):rest -> (rest, fn)
+					(mkForeignLabel lbl call_size False IsFunction)))
+	   DynamicTarget    ->  case args of (CmmKinded fn _):rest -> (rest, fn)
 
 	-- in the stdcall calling convention, the symbol needs @size appended
 	-- to it, where size is the total number of bytes of arguments.  We
 	-- attach this info to the CLabel here, and the CLabel pretty printer
 	-- will generate the suffix when the label is printed.
       call_size
-	| StdCallConv <- cconv = Just (sum (map (arg_size.cmmExprRep.fst) args))
+	| StdCallConv <- cconv = Just (sum (map (arg_size.cmmExprRep.kindlessCmm) args))
 	| otherwise            = Nothing
 
 	-- ToDo: this might not be correct for 64-bit API
@@ -106,9 +107,9 @@ emitForeignCall _ (DNCall _) _ _
 -- alternative entry point, used by CmmParse
 emitForeignCall'
 	:: Safety
-	-> CmmHintFormals	-- where to put the results
+	-> CmmFormals	-- where to put the results
 	-> CmmCallTarget	-- the op
-	-> [(CmmExpr,MachHint)] -- arguments
+	-> [CmmKinded CmmExpr] -- arguments
 	-> Maybe [GlobalReg]	-- live vars, in case we need to save them
         -> C_SRT                -- the SRT of the calls continuation
         -> CmmReturnInfo
@@ -122,7 +123,7 @@ emitForeignCall' safety results target args vols srt ret
     stmtsC caller_load
 
   | otherwise = do
-    -- Both 'id' and 'new_base' are KindNonPtr because they're
+    -- Both 'id' and 'new_base' are GCKindNonPtr because they're
     -- RTS only objects and are not subject to garbage collection
     id <- newNonPtrTemp wordRep
     new_base <- newNonPtrTemp (cmmRegRep (CmmGlobal BaseReg))
@@ -137,13 +138,13 @@ emitForeignCall' safety results target args vols srt ret
     -- and the CPS will will be the one to convert that
     -- to this sequence of three CmmUnsafe calls.
     stmtC (CmmCall (CmmCallee suspendThread CCallConv) 
-			[ (id,PtrHint) ]
-			[ (CmmReg (CmmGlobal BaseReg), PtrHint) ] 
+			[ CmmKinded id PtrHint ]
+			[ CmmKinded (CmmReg (CmmGlobal BaseReg)) PtrHint ] 
 			CmmUnsafe ret)
     stmtC (CmmCall temp_target results temp_args CmmUnsafe ret)
     stmtC (CmmCall (CmmCallee resumeThread CCallConv) 
-			[ (new_base, PtrHint) ]
-			[ (CmmReg (CmmLocal id), PtrHint) ]
+			[ CmmKinded new_base PtrHint ]
+			[ CmmKinded (CmmReg (CmmLocal id)) PtrHint ]
 			CmmUnsafe ret)
     -- Assign the result to BaseReg: we
     -- might now have a different Capability!
@@ -151,8 +152,8 @@ emitForeignCall' safety results target args vols srt ret
     stmtsC caller_load
     emitLoadThreadState
 
-suspendThread = CmmLit (CmmLabel (mkRtsCodeLabel SLIT("suspendThread")))
-resumeThread  = CmmLit (CmmLabel (mkRtsCodeLabel SLIT("resumeThread")))
+suspendThread = CmmLit (CmmLabel (mkRtsCodeLabel (sLit "suspendThread")))
+resumeThread  = CmmLit (CmmLabel (mkRtsCodeLabel (sLit "resumeThread")))
 
 
 -- we might need to load arguments into temporaries before
@@ -163,9 +164,9 @@ resumeThread  = CmmLit (CmmLabel (mkRtsCodeLabel SLIT("resumeThread")))
 -- This is a HACK; really it should be done in the back end, but
 -- it's easier to generate the temporaries here.
 load_args_into_temps = mapM arg_assign_temp
-  where arg_assign_temp (e,hint) = do
+  where arg_assign_temp (CmmKinded e hint) = do
 	   tmp <- maybe_assign_temp e
-	   return (tmp,hint)
+	   return (CmmKinded tmp hint)
 	
 load_target_into_temp (CmmCallee expr conv) = do 
   tmp <- maybe_assign_temp expr
