@@ -53,6 +53,7 @@ module CgUtils (
 #include "HsVersions.h"
 #include "../includes/MachRegs.h"
 
+import BlockId
 import CgMonad
 import TyCon
 import DataCon
@@ -110,7 +111,7 @@ mkSimpleLit (MachWord i)      = CmmInt i wordRep
 mkSimpleLit (MachWord64 i)    = CmmInt i I64
 mkSimpleLit (MachFloat r)     = CmmFloat r F32
 mkSimpleLit (MachDouble r)    = CmmFloat r F64
-mkSimpleLit (MachLabel fs ms) = CmmLabel (mkForeignLabel fs ms is_dyn)
+mkSimpleLit (MachLabel fs ms fod) = CmmLabel (mkForeignLabel fs ms is_dyn fod)
 			      where
 				is_dyn = False	-- ToDo: fix me
 	
@@ -333,24 +334,24 @@ emitIfThenElse cond then_part else_part
        ; labelC join_id
        }
 
-emitRtsCall :: LitString -> [(CmmExpr,MachHint)] -> Bool -> Code
+emitRtsCall :: LitString -> [CmmKinded CmmExpr] -> Bool -> Code
 emitRtsCall fun args safe = emitRtsCall' [] fun args Nothing safe
    -- The 'Nothing' says "save all global registers"
 
-emitRtsCallWithVols :: LitString -> [(CmmExpr,MachHint)] -> [GlobalReg] -> Bool -> Code
+emitRtsCallWithVols :: LitString -> [CmmKinded CmmExpr] -> [GlobalReg] -> Bool -> Code
 emitRtsCallWithVols fun args vols safe
    = emitRtsCall' [] fun args (Just vols) safe
 
 emitRtsCallWithResult :: LocalReg -> MachHint -> LitString
-	-> [(CmmExpr,MachHint)] -> Bool -> Code
+	-> [CmmKinded CmmExpr] -> Bool -> Code
 emitRtsCallWithResult res hint fun args safe
-   = emitRtsCall' [(res,hint)] fun args Nothing safe
+   = emitRtsCall' [CmmKinded res hint] fun args Nothing safe
 
 -- Make a call to an RTS C procedure
 emitRtsCall'
-   :: CmmHintFormals
+   :: CmmFormals
    -> LitString
-   -> [(CmmExpr,MachHint)]
+   -> [CmmKinded CmmExpr]
    -> Maybe [GlobalReg]
    -> Bool -- True <=> CmmSafe call
    -> Code
@@ -438,7 +439,7 @@ get_Regtable_addr_from_offset rep offset =
 #endif
 
 
--- | Returns 'True' if this global register is stored in a caller-saves
+-- | Returns @True@ if this global register is stored in a caller-saves
 -- machine register.
 
 callerSaves :: GlobalReg -> Bool
@@ -543,10 +544,8 @@ baseRegOffset CurrentNursery	  = oFFSET_StgRegTable_rCurrentNursery
 baseRegOffset HpAlloc		  = oFFSET_StgRegTable_rHpAlloc
 baseRegOffset GCEnter1		  = oFFSET_stgGCEnter1
 baseRegOffset GCFun		  = oFFSET_stgGCFun
-#ifdef DEBUG
 baseRegOffset BaseReg		  = panic "baseRegOffset:BaseReg"
 baseRegOffset _			  = panic "baseRegOffset:other"
-#endif
 
 
 -------------------------------------------------------------------------
@@ -621,10 +620,10 @@ assignPtrTemp e
 			    ; return (CmmReg (CmmLocal reg)) }
 
 newNonPtrTemp :: MachRep -> FCode LocalReg
-newNonPtrTemp rep = do { uniq <- newUnique; return (LocalReg uniq rep KindNonPtr) }
+newNonPtrTemp rep = do { uniq <- newUnique; return (LocalReg uniq rep GCKindNonPtr) }
 
 newPtrTemp :: MachRep -> FCode LocalReg
-newPtrTemp rep = do { uniq <- newUnique; return (LocalReg uniq rep KindPtr) }
+newPtrTemp rep = do { uniq <- newUnique; return (LocalReg uniq rep GCKindPtr) }
 
 
 -------------------------------------------------------------------------
@@ -905,7 +904,7 @@ doSimultaneously1 vertices
 	edges_from stmt1 = [ key2 | (key2, stmt2) <- vertices, 
 				    stmt1 `mustFollow` stmt2
 			   ]
-	components = stronglyConnComp edges
+	components = stronglyConnCompFromEdgedVertices edges
 
 	-- do_components deal with one strongly-connected component
 	-- Not cyclic, or singleton?  Just do it
@@ -994,6 +993,7 @@ getSRTInfo = do
     -- TODO: Should we panic in this case?
     -- Someone obviously thinks there should be an SRT
     NoSRT -> return NoC_SRT
+    SRTEntries {} -> panic "getSRTInfo: SRTEntries.  Perhaps you forgot to run SimplStg?"
     SRT off len bmp
       | len > hALF_WORD_SIZE_IN_BITS || bmp == [fromIntegral srt_escape]
       -> do id <- newUnique
