@@ -2,12 +2,12 @@
 -- |
 -- Module      :  Distribution.Simple.JHC
 -- Copyright   :  Isaac Jones 2003-2006
--- 
--- Maintainer  :  Isaac Jones <ijones@syntaxpolice.org>
--- Stability   :  alpha
+--
+-- Maintainer  :  cabal-devel@haskell.org
 -- Portability :  portable
 --
--- Build and install functionality for the JHC compiler.
+-- This module contains most of the JHC-specific code for configuring, building
+-- and installing packages.
 
 {- Copyright (c) 2003-2005, Isaac Jones
 All rights reserved.
@@ -41,36 +41,47 @@ THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. -}
 
 module Distribution.Simple.JHC (
-	configure, getInstalledPackages, build, installLib, installExe
+        configure, getInstalledPackages, build, installLib, installExe
  ) where
 
-import Distribution.PackageDescription
-				( PackageDescription(..), BuildInfo(..),
-				  withLib,
-				  Executable(..), withExe, Library(..),
-				  libModules, hcOptions )
+import Distribution.PackageDescription as PD
+                                ( PackageDescription(..), BuildInfo(..),
+                                  withLib,
+                                  Executable(..), withExe, Library(..),
+                                  libModules, hcOptions )
+import Distribution.InstalledPackageInfo
+                                ( InstalledPackageInfo, emptyInstalledPackageInfo )
+import qualified Distribution.InstalledPackageInfo as InstalledPackageInfo
+                                ( InstalledPackageInfo_(package) )
+import Distribution.Simple.PackageIndex (PackageIndex)
+import qualified Distribution.Simple.PackageIndex as PackageIndex
 import Distribution.Simple.LocalBuildInfo
-				( LocalBuildInfo(..), 
-				  autogenModulesDir )
-import Distribution.Simple.Compiler ( Compiler(..), CompilerFlavor(..), Flag,
-                                  PackageDB, extensionsToFlags )
+                                ( LocalBuildInfo(..) )
+import Distribution.Simple.BuildPaths
+                                ( autogenModulesDir, exeExtension )
+import Distribution.Simple.Compiler
+         ( CompilerFlavor(..), CompilerId(..), Compiler(..)
+         , PackageDB, Flag, extensionsToFlags )
 import Language.Haskell.Extension (Extension(..))
 import Distribution.Simple.Program     ( ConfiguredProgram(..), jhcProgram,
                                   ProgramConfiguration, userMaybeSpecifyPath,
                                   requireProgram, lookupProgram,
                                   rawSystemProgram, rawSystemProgramStdoutConf )
-import Distribution.Version	( VersionRange(AnyVersion) )
-import Distribution.Package  	( PackageIdentifier(..), showPackageId,
-                                  parsePackageId )
-import Distribution.Simple.Utils( createDirectoryIfMissingVerbose,
-                                  copyFileVerbose, exeExtension, die, info )
+import Distribution.Version     ( VersionRange(AnyVersion) )
+import Distribution.Package
+         ( Package(..) )
+import Distribution.Simple.Utils
+        ( createDirectoryIfMissingVerbose, copyFileVerbose, writeFileAtomic
+        , die, info, intercalate )
 import System.FilePath          ( (</>) )
 import Distribution.Verbosity
+import Distribution.Text
+         ( Text(parse), display )
 import Distribution.Compat.ReadP
     ( readP_to_S, many, skipSpaces )
 
-import Data.List		( nub, intersperse )
-import Data.Char		( isSpace )
+import Data.List                ( nub )
+import Data.Char                ( isSpace )
 
 
 
@@ -86,8 +97,7 @@ configure verbosity hcPath _hcPkgPath conf = do
 
   let Just version = programVersion jhcProg
       comp = Compiler {
-        compilerFlavor         = JHC,
-        compilerId             = PackageIdentifier "jhc" version,
+        compilerId             = CompilerId JHC version,
         compilerExtensions     = jhcLanguageExtensions
       }
   return (comp, conf')
@@ -102,11 +112,15 @@ jhcLanguageExtensions =
     ]
 
 getInstalledPackages :: Verbosity -> PackageDB -> ProgramConfiguration
-                    -> IO [PackageIdentifier]
+                    -> IO (PackageIndex InstalledPackageInfo)
 getInstalledPackages verbosity _packagedb conf = do
    str <- rawSystemProgramStdoutConf verbosity jhcProgram conf ["--list-libraries"]
-   case pCheck (readP_to_S (many (skipSpaces >> parsePackageId)) str) of
-     [ps] -> return ps
+   case pCheck (readP_to_S (many (skipSpaces >> parse)) str) of
+     [ps] -> return $ PackageIndex.fromList
+                    [ emptyInstalledPackageInfo {
+                        InstalledPackageInfo.package = p
+                      }
+                    | p <- ps ]
      _    -> die "cannot parse package list"
   where
     pCheck :: [(a, [Char])] -> [a]
@@ -124,11 +138,12 @@ build pkg_descr lbi verbosity = do
       info verbosity "Building library..."
       let libBi = libBuildInfo lib
       let args  = constructJHCCmdLine lbi libBi (buildDir lbi) verbosity
-      rawSystemProgram verbosity jhcProg (["-c"] ++ args ++ libModules pkg_descr)
-      let pkgid = showPackageId (package pkg_descr)
+      rawSystemProgram verbosity jhcProg $
+        ["-c"] ++ args ++ map display (libModules pkg_descr)
+      let pkgid = display (packageId pkg_descr)
           pfile = buildDir lbi </> "jhc-pkg.conf"
           hlfile= buildDir lbi </> (pkgid ++ ".hl")
-      writeFile pfile $ jhcPkgConf pkg_descr
+      writeFileAtomic pfile $ jhcPkgConf pkg_descr
       rawSystemProgram verbosity jhcProg ["--build-hl="++pfile, "-o", hlfile]
   withExe pkg_descr $ \exe -> do
       info verbosity ("Building executable "++exeName exe)
@@ -141,32 +156,34 @@ constructJHCCmdLine :: LocalBuildInfo -> BuildInfo -> FilePath -> Verbosity -> [
 constructJHCCmdLine lbi bi _odir verbosity =
         (if verbosity >= deafening then ["-v"] else [])
      ++ extensionsToFlags (compiler lbi) (extensions bi)
-     ++ hcOptions JHC (options bi)
+     ++ hcOptions JHC bi
      ++ ["--noauto","-i-"]
-     ++ ["-i", autogenModulesDir lbi]
      ++ concat [["-i", l] | l <- nub (hsSourceDirs bi)]
-     ++ ["-optc" ++ opt | opt <- ccOptions bi]
-     ++ (concat [ ["-p", showPackageId pkg] | pkg <- packageDeps lbi ])
+     ++ ["-i", autogenModulesDir lbi]
+     ++ ["-optc" ++ opt | opt <- PD.ccOptions bi]
+     ++ (concat [ ["-p", display pkg] | pkg <- packageDeps lbi ])
 
 jhcPkgConf :: PackageDescription -> String
 jhcPkgConf pd =
   let sline name sel = name ++ ": "++sel pd
       Just lib = library pd
-      comma f l = concat $ intersperse "," $ map f l
-  in unlines [sline "name" (showPackageId . package)
-             ,"exposed-modules: " ++ (comma id (exposedModules lib))
-             ,"hidden-modules: " ++ (comma id (otherModules $ libBuildInfo lib))
+      comma = intercalate "," . map display
+  in unlines [sline "name" (display . packageId)
+             ,"exposed-modules: " ++ (comma (PD.exposedModules lib))
+             ,"hidden-modules: " ++ (comma (otherModules $ libBuildInfo lib))
              ]
 
 installLib :: Verbosity -> FilePath -> FilePath -> PackageDescription -> Library -> IO ()
 installLib verb dest build_dir pkg_descr _ = do
-    let p = showPackageId (package pkg_descr)++".hl"
+    let p = display (packageId pkg_descr)++".hl"
     createDirectoryIfMissingVerbose verb True dest
     copyFileVerbose verb (build_dir </> p) (dest </> p)
 
-installExe :: Verbosity -> FilePath -> FilePath -> PackageDescription -> Executable -> IO ()
-installExe verb dest build_dir _ exe = do
-    let out   = exeName exe </> exeExtension
+installExe :: Verbosity -> FilePath -> FilePath -> (FilePath,FilePath) -> PackageDescription -> Executable -> IO ()
+installExe verb dest build_dir (progprefix,progsuffix) _ exe = do
+    let exe_name = exeName exe
+        src = exe_name </> exeExtension
+        out   = (progprefix ++ exe_name ++ progsuffix) </> exeExtension
     createDirectoryIfMissingVerbose verb True dest
-    copyFileVerbose verb (build_dir </> out) (dest </> out)
+    copyFileVerbose verb (build_dir </> src) (dest </> out)
 
