@@ -1,15 +1,24 @@
-{-# OPTIONS -cpp #-}
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  Distribution.Simple.Program
 -- Copyright   :  Isaac Jones 2006
--- 
--- Maintainer  :  Isaac Jones <ijones@syntaxpolice.org>
--- Stability   :  alpha
--- Portability :  GHC, Hugs
 --
--- Explanation: A program is basically a name, a location, and some
--- arguments.
+-- Maintainer  :  cabal-devel@haskell.org
+-- Portability :  portable
+--
+-- This provides an abstraction which deals with configuring and running
+-- programs. A 'Program' is a static notion of a known program. A
+-- 'ConfiguredProgram' is a 'Program' that has been found on the current
+-- machine and is ready to be run (possibly with some user-supplied default
+-- args). Configuring a program involves finding its location and if necessary
+-- finding its version. There is also a 'ProgramConfiguration' type which holds
+-- configured and not-yet configured programs. It is the parameter to lots of
+-- actions elsewhere in Cabal that need to look up and run programs. If we had
+-- a Cabal monad, the 'ProgramConfiguration' would probably be a reader or
+-- state component of it. 
+--
+-- The module also defines all the known built-in 'Program's and the
+-- 'defaultProgramConfiguration' which contains them all.
 --
 -- One nice thing about using it is that any program that is
 -- registered with Cabal will get some \"configure\" and \".cabal\"
@@ -45,15 +54,22 @@ module Distribution.Simple.Program (
     , ProgramConfiguration
     , emptyProgramConfiguration
     , defaultProgramConfiguration
+    , restoreProgramConfiguration
     , addKnownProgram
+    , addKnownPrograms
     , lookupKnownProgram
     , knownPrograms
     , userSpecifyPath
+    , userSpecifyPaths
     , userMaybeSpecifyPath
     , userSpecifyArgs
+    , userSpecifyArgss
+    , userSpecifiedArgs
     , lookupProgram
     , updateProgram
+    , configureProgram
     , configureAllKnownPrograms
+    , reconfigurePrograms
     , requireProgram
     , rawSystemProgramConf
     , rawSystemProgramStdoutConf
@@ -66,8 +82,10 @@ module Distribution.Simple.Program (
     , jhcProgram
     , hugsProgram
     , ffihugsProgram
+    , gccProgram
     , ranlibProgram
     , arProgram
+    , stripProgram
     , happyProgram
     , alexProgram
     , hsc2hsProgram
@@ -79,36 +97,43 @@ module Distribution.Simple.Program (
     , ldProgram
     , tarProgram
     , cppProgram
-    , pfesetupProgram
     , pkgConfigProgram
     ) where
 
-import qualified Distribution.Compat.Map as Map
-import Distribution.Compat.Directory (findExecutable)
-import Distribution.Compat.TempFile (withTempFile)
-import Distribution.Simple.Utils (die, debug, warn, rawSystemExit,
-                                  rawSystemStdout, rawSystemStdout')
-import Distribution.Version (Version(..), readVersion, showVersion,
-                             VersionRange(..), withinRange, showVersionRange)
+import Data.List  (foldl')
+import Data.Maybe (catMaybes)
+import qualified Data.Map as Map
+
+import Distribution.Simple.Utils
+         (die, debug, warn, rawSystemExit, rawSystemStdout)
+import Distribution.Version
+         ( Version(..), VersionRange(AnyVersion), withinRange )
+import Distribution.Text
+         ( simpleParse, display )
 import Distribution.Verbosity
-import System.Directory (doesFileExist, removeFile)
-import System.FilePath  (dropExtension)
-import System.IO.Error (try)
+import System.Directory
+         ( doesFileExist, findExecutable )
 import Control.Monad (join, foldM)
-import Control.Exception as Exception (catch)
+import Distribution.Compat.Exception (catchExit, catchIO)
 
 -- | Represents a program which can be configured.
 data Program = Program {
         -- | The simple name of the program, eg. ghc
         programName :: String,
-        
+
         -- | A function to search for the program if it's location was not
-        -- specified by the user. Usually this will just be a 
+        -- specified by the user. Usually this will just be a
         programFindLocation :: Verbosity -> IO (Maybe FilePath),
-        
+
         -- | Try to find the version of the program. For many programs this is
         -- not possible or is not necessary so it's ok to return Nothing.
-        programFindVersion :: Verbosity -> FilePath -> IO (Maybe Version)
+        programFindVersion :: Verbosity -> FilePath -> IO (Maybe Version),
+
+        -- | A function to do any additional configuration after we have
+        -- located the program (and perhaps identified its version). It is
+        -- allowed to return additional flags that will be passed to the
+        -- program on every invocation.
+        programPostConf :: Verbosity -> ConfiguredProgram -> IO [ProgArg]
     }
 
 type ProgArg = String
@@ -116,7 +141,7 @@ type ProgArg = String
 data ConfiguredProgram = ConfiguredProgram {
         -- | Just the name again
         programId :: String,
-        
+
         -- | The version of this program, if it is known.
         programVersion :: Maybe Version,
 
@@ -126,8 +151,8 @@ data ConfiguredProgram = ConfiguredProgram {
         programArgs :: [ProgArg],
 
         -- | Location of the program. eg. @\/usr\/bin\/ghc-6.4@
-        programLocation :: ProgramLocation    
-    } deriving (Read, Show)
+        programLocation :: ProgramLocation
+    } deriving (Read, Show, Eq)
 
 -- | Where a program was found. Also tells us whether it's specifed by user or
 -- not.  This includes not just the path, but the program as well.
@@ -137,7 +162,7 @@ data ProgramLocation
       -- eg. --ghc-path=\/usr\/bin\/ghc-6.6
     | FoundOnSystem { locationPath :: FilePath }
       -- ^The location of the program, as located by searching PATH.
-      deriving (Read, Show)
+      deriving (Read, Show, Eq)
 
 -- ------------------------------------------------------------
 -- * Programs functions
@@ -155,8 +180,12 @@ programPath = locationPath . programLocation
 -- > simpleProgram "foo" { programFindLocation = ... , programFindVersion ... }
 --
 simpleProgram :: String -> Program
-simpleProgram name = 
-  Program name (findProgramOnPath name) (\_ _ -> return Nothing)
+simpleProgram name = Program {
+    programName         = name,
+    programFindLocation = findProgramOnPath name,
+    programFindVersion  = \_ _ -> return Nothing,
+    programPostConf     = \_ _ -> return []
+  }
 
 -- | Look for a program on the path.
 findProgramOnPath :: FilePath -> Verbosity -> IO (Maybe FilePath)
@@ -180,12 +209,14 @@ findProgramVersion :: ProgArg            -- ^ version args
                    -> IO (Maybe Version)
 findProgramVersion versionArg selectVersion verbosity path = do
   str <- rawSystemStdout verbosity path [versionArg]
-         `Exception.catch` \_ -> return ""
-  let version = readVersion (selectVersion str)
+         `catchIO`   (\_ -> return "")
+         `catchExit` (\_ -> return "")
+  let version :: Maybe Version
+      version = simpleParse (selectVersion str)
   case version of
       Nothing -> warn verbosity $ "cannot determine version of " ++ path
                                ++ " :\n" ++ show str
-      Just v  -> debug verbosity $ path ++ " is version " ++ showVersion v
+      Just v  -> debug verbosity $ path ++ " is version " ++ display v
   return version
 
 -- ------------------------------------------------------------
@@ -213,7 +244,7 @@ emptyProgramConfiguration = ProgramConfiguration Map.empty Map.empty
 
 defaultProgramConfiguration :: ProgramConfiguration
 defaultProgramConfiguration =
-  foldl (flip addKnownProgram) emptyProgramConfiguration builtinPrograms
+  restoreProgramConfiguration builtinPrograms emptyProgramConfiguration
 
 -- internal helpers:
 updateUnconfiguredProgs :: (UnconfiguredProgs -> UnconfiguredProgs)
@@ -237,20 +268,36 @@ instance Read ProgramConfiguration where
     [ (emptyProgramConfiguration { configuredProgs = Map.fromList s' }, r)
     | (s', r) <- readsPrec p s ]
 
+-- | The Read\/Show instance does not preserve all the unconfigured 'Programs'
+-- because 'Program' is not in Read\/Show because it contains functions. So to
+-- fully restore a deserialised 'ProgramConfiguration' use this function to add
+-- back all the known 'Program's.
+--
+-- * It does not add the default programs, but you probably want them, use
+--   'builtinPrograms' in addition to any extra you might need.
+--
+restoreProgramConfiguration :: [Program] -> ProgramConfiguration
+                                         -> ProgramConfiguration
+restoreProgramConfiguration = addKnownPrograms
+
 -- -------------------------------
 -- Managing unconfigured programs
 
 -- | Add a known program that we may configure later
 addKnownProgram :: Program -> ProgramConfiguration -> ProgramConfiguration
 addKnownProgram prog = updateUnconfiguredProgs $
-  Map.insert (programName prog) (prog, Nothing, [])
+  Map.insertWith combine (programName prog) (prog, Nothing, [])
+  where combine _ (_, path, args) = (prog, path, args)
+
+addKnownPrograms :: [Program] -> ProgramConfiguration -> ProgramConfiguration
+addKnownPrograms progs conf = foldl' (flip addKnownProgram) conf progs
 
 lookupKnownProgram :: String -> ProgramConfiguration -> Maybe Program
 lookupKnownProgram name =
   fmap (\(p,_,_)->p) . Map.lookup name . unconfiguredProgs
 
 knownPrograms :: ProgramConfiguration -> [(Program, Maybe ConfiguredProgram)]
-knownPrograms conf = 
+knownPrograms conf =
   [ (p,p') | (p,_,_) <- Map.elems (unconfiguredProgs conf)
            , let p' = Map.lookup (programName p) (configuredProgs conf) ]
 
@@ -283,10 +330,30 @@ userSpecifyArgs name args' =
       (flip Map.update name $
          \prog -> Just prog { programArgs = programArgs prog ++ args' })
 
+-- | Like 'userSpecifyPath' but for a list of progs and their paths.
+--
+userSpecifyPaths :: [(String, FilePath)]
+                 -> ProgramConfiguration
+                 -> ProgramConfiguration
+userSpecifyPaths paths conf =
+  foldl' (\conf' (prog, path) -> userSpecifyPath prog path conf') conf paths
+
+-- | Like 'userSpecifyPath' but for a list of progs and their args.
+--
+userSpecifyArgss :: [(String, [ProgArg])]
+                 -> ProgramConfiguration
+                 -> ProgramConfiguration
+userSpecifyArgss argss conf =
+  foldl' (\conf' (prog, args) -> userSpecifyArgs prog args conf') conf argss
+
+-- | Get the path that has been previously specified for a program, if any.
+--
 userSpecifiedPath :: Program -> ProgramConfiguration -> Maybe FilePath
 userSpecifiedPath prog =
   join . fmap (\(_,p,_)->p) . Map.lookup (programName prog) . unconfiguredProgs
 
+-- | Get any extra args that have been previously specified for a program.
+--
 userSpecifiedArgs :: Program -> ProgramConfiguration -> [ProgArg]
 userSpecifiedArgs prog =
   maybe [] (\(_,_,as)->as) . Map.lookup (programName prog) . unconfiguredProgs
@@ -309,7 +376,7 @@ updateProgram prog = updateConfiguredProgs $
 
 -- | Try to configure a specific program. If the program is already included in
 -- the colleciton of unconfigured programs then we use any user-supplied
--- location and arguments. If the program gets configured sucessfully it gets 
+-- location and arguments. If the program gets configured sucessfully it gets
 -- added to the configured collection.
 --
 -- Note that it is not a failure if the program cannot be configured. It's only
@@ -319,7 +386,7 @@ updateProgram prog = updateConfiguredProgs $
 -- The reason for it not being a failure at this stage is that we don't know up
 -- front all the programs we will need, so we try to configure them all.
 -- To verify that a program was actually sucessfully configured use
--- 'requireProgram'. 
+-- 'requireProgram'.
 --
 configureProgram :: Verbosity
                  -> Program
@@ -336,28 +403,60 @@ configureProgram verbosity prog conf = do
         then return (Just (UserSpecified path))
         else findProgramOnPath path verbosity
          >>= maybe (die notFound) (return . Just . UserSpecified)
-      where notFound = "Cannot find " ++ name ++ " at "
-                     ++ path ++ " or on the path"
+      where notFound = "Cannot find the program '" ++ name ++ "' at '"
+                     ++ path ++ "' or on the path"
   case maybeLocation of
     Nothing -> return conf
     Just location -> do
       version <- programFindVersion prog verbosity (locationPath location)
-      let configuredProg = ConfiguredProgram {
+      let configuredProg    = ConfiguredProgram {
             programId       = name,
             programVersion  = version,
             programArgs     = userSpecifiedArgs prog conf,
             programLocation = location
           }
-      return (updateConfiguredProgs (Map.insert name configuredProg) conf)
+      extraArgs <- programPostConf prog verbosity configuredProg
+      let configuredProg'   = configuredProg {
+            programArgs     = extraArgs ++ programArgs configuredProg
+          }
+      return (updateConfiguredProgs (Map.insert name configuredProg') conf)
+
+-- | Configure a bunch of programs using 'configureProgram'. Just a 'foldM'.
+configurePrograms :: Verbosity
+                  -> [Program]
+                  -> ProgramConfiguration
+                  -> IO ProgramConfiguration
+configurePrograms verbosity progs conf =
+  foldM (flip (configureProgram verbosity)) conf progs
 
 -- | Try to configure all the known programs that have not yet been configured.
 configureAllKnownPrograms :: Verbosity
-                  -> ProgramConfiguration
-                  -> IO ProgramConfiguration
+                          -> ProgramConfiguration
+                          -> IO ProgramConfiguration
 configureAllKnownPrograms verbosity conf =
-  foldM (flip (configureProgram verbosity)) conf
-    [ prog | (prog,_,_) <- Map.elems (unconfiguredProgs conf
-                     `Map.difference` configuredProgs conf) ]
+  configurePrograms verbosity
+    [ prog | (prog,_,_) <- Map.elems notYetConfigured ] conf
+  where
+    notYetConfigured = unconfiguredProgs conf
+      `Map.difference` configuredProgs conf
+
+-- | reconfigure a bunch of programs given new user-specified args. It takes
+-- the same inputs as 'userSpecifyPath' and 'userSpecifyArgs' and for all progs
+-- with a new path it calls 'configureProgram'.
+--
+reconfigurePrograms :: Verbosity
+                    -> [(String, FilePath)]
+                    -> [(String, [ProgArg])]
+                    -> ProgramConfiguration
+                    -> IO ProgramConfiguration
+reconfigurePrograms verbosity paths argss conf = do
+  configurePrograms verbosity progs
+   . userSpecifyPaths paths
+   . userSpecifyArgss argss
+   $ conf
+
+  where
+    progs = catMaybes [ lookupKnownProgram name conf | (name,_) <- paths ]
 
 -- | Check that a program is configured and available to be run.
 --
@@ -369,12 +468,12 @@ configureAllKnownPrograms verbosity conf =
 requireProgram :: Verbosity -> Program -> VersionRange -> ProgramConfiguration
                -> IO (ConfiguredProgram, ProgramConfiguration)
 requireProgram verbosity prog range conf = do
-  
+
   -- If it's not already been configured, try to configure it now
   conf' <- case lookupProgram prog conf of
     Nothing -> configureProgram verbosity prog conf
     Just _  -> return conf
-  
+
   case lookupProgram prog conf' of
     Nothing                           -> die notFound
     Just configuredProg
@@ -390,13 +489,13 @@ requireProgram verbosity prog range conf = do
                       ++ " is required but it could not be found."
         badVersion v l = programName prog ++ versionRequirement
                       ++ " is required but the version found at "
-                      ++ locationPath l ++ " is version " ++ showVersion v
+                      ++ locationPath l ++ " is version " ++ display v
         noVersion l    = programName prog ++ versionRequirement
                       ++ " is required but the version of "
                       ++ locationPath l ++ " could not be determined."
         versionRequirement
           | range == AnyVersion = ""
-          | otherwise           = " version " ++ showVersionRange range
+          | otherwise           = " version " ++ display range
 
 -- ------------------------------------------------------------
 -- * Running programs
@@ -466,10 +565,11 @@ builtinPrograms =
     , c2hsProgram
     , cpphsProgram
     , greencardProgram
-    , pfesetupProgram
     -- platform toolchain
+    , gccProgram
     , ranlibProgram
     , arProgram
+    , stripProgram
     , ldProgram
     , tarProgram
     -- configuration tools
@@ -492,11 +592,20 @@ ghcPkgProgram = (simpleProgram "ghc-pkg") {
   }
 
 nhcProgram :: Program
-nhcProgram = simpleProgram "nhc98"
+nhcProgram = (simpleProgram "nhc98") {
+    programFindVersion = findProgramVersion "--version" $ \str ->
+      -- Invoking "nhc98 --version" gives a string like
+      -- "/usr/local/bin/nhc98: v1.20 (2007-11-22)"
+      case words str of
+        (_:('v':ver):_) -> ver
+        _               -> ""
+  }
 
 hmakeProgram :: Program
 hmakeProgram = (simpleProgram "hmake") {
     programFindVersion = findProgramVersion "--version" $ \str ->
+    -- Invoking "hmake --version" gives a string line
+    -- "/usr/local/bin/hmake: 3.13 (2006-11-01)"
       case words str of
         (_:ver:_) -> ver
         _         -> ""
@@ -505,6 +614,9 @@ hmakeProgram = (simpleProgram "hmake") {
 jhcProgram :: Program
 jhcProgram = (simpleProgram "jhc") {
     programFindVersion = findProgramVersion "--version" $ \str ->
+    -- invoking "jhc --version" gives a string like
+    -- "jhc 0.3.20080208 (wubgipkamcep-2)
+    -- compiled by ghc-6.8 on a x86_64 running linux"
       case words str of
         (_:ver:_) -> ver
         _         -> ""
@@ -537,6 +649,10 @@ alexProgram = (simpleProgram "alex") {
         _           -> ""
   }
 
+gccProgram :: Program
+gccProgram = (simpleProgram "gcc") {
+    programFindVersion = findProgramVersion "-dumpversion" id
+  }
 
 ranlibProgram :: Program
 ranlibProgram = simpleProgram "ranlib"
@@ -544,33 +660,17 @@ ranlibProgram = simpleProgram "ranlib"
 arProgram :: Program
 arProgram = simpleProgram "ar"
 
+stripProgram :: Program
+stripProgram = simpleProgram "strip"
+
 hsc2hsProgram :: Program
 hsc2hsProgram = (simpleProgram "hsc2hs") {
-    programFindVersion = \verbosity path -> do
-      maybeVersion <- findProgramVersion "--version" (\str ->
+    programFindVersion =
+      findProgramVersion "--version" $ \str ->
         -- Invoking "hsc2hs --version" gives a string like "hsc2hs version 0.66"
         case words str of
           (_:_:ver:_) -> ver
-          _           -> "") verbosity path
-
-      -- It turns out that it's important to know if hsc2hs is using gcc or ghc
-      -- as it's C compiler since this affects how we escape C options.
-      -- So here's a cunning hack, we make a temp .hsc file and call:
-      -- hsch2s tmp.hsc --cflag=--version
-      -- which passes --version through to ghc/gcc and we look at the result
-      -- to see if it was indeed ghc or not.
-      case maybeVersion of
-        Nothing -> return Nothing
-	Just version ->
-          withTempFile "dist" "hsc" $ \hsc -> do
-	    writeFile hsc ""
-	    (str, _) <- rawSystemStdout' verbosity path [hsc, "--cflag=--version"]
-	    try $ removeFile (dropExtension hsc ++ "_hsc_make.c")
-	    case words str of
-	      (_:"Glorious":"Glasgow":"Haskell":_)
-	        -> return $ Just version { versionTags = ["ghc"] }
-	      _ -> return $ Just version
-
+          _           -> ""
   }
 
 c2hsProgram :: Program
@@ -619,9 +719,6 @@ tarProgram = simpleProgram "tar"
 cppProgram :: Program
 cppProgram = simpleProgram "cpp"
 
-pfesetupProgram :: Program
-pfesetupProgram = simpleProgram "pfesetup"
-
 pkgConfigProgram :: Program
 pkgConfigProgram = (simpleProgram "pkg-config") {
     programFindVersion = findProgramVersion "--version" id
