@@ -1,10 +1,10 @@
-{-# OPTIONS -cpp -fglasgow-exts #-}
+{-# OPTIONS -cpp #-}
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  Data.Sequence
 -- Copyright   :  (c) Ross Paterson 2005
 -- License     :  BSD-style
--- Maintainer  :  ross@soi.city.ac.uk
+-- Maintainer  :  libraries@haskell.org
 -- Stability   :  experimental
 -- Portability :  portable
 --
@@ -75,13 +75,16 @@ import Control.Monad (MonadPlus(..))
 import Data.Monoid (Monoid(..))
 import Data.Foldable
 import Data.Traversable
-import Data.Typeable
+#ifndef __GLASGOW_HASKELL__
+import Data.Typeable (Typeable, typeOf, typeOfDefault)
+#endif
+import Data.Typeable (TyCon, Typeable1(..), mkTyCon, mkTyConApp )
 
 #ifdef __GLASGOW_HASKELL__
 import Text.Read (Lexeme(Ident), lexP, parens, prec,
 	readPrec, readListPrec, readListPrecDefault)
-import Data.Generics.Basics (Data(..), Fixity(..),
-			constrIndex, mkConstr, mkDataType)
+import Data.Data (Data(..), DataType, Constr, Fixity(..),
+                             mkConstr, mkDataType, constrIndex, gcast1)
 #endif
 
 #if TESTING
@@ -183,8 +186,11 @@ instance Data a => Data (Seq a) where
 
 	dataCast1 f	= gcast1 f
 
+emptyConstr, consConstr :: Constr
 emptyConstr = mkConstr seqDataType "empty" [] Prefix
 consConstr  = mkConstr seqDataType "<|" [] Infix
+
+seqDataType :: DataType
 seqDataType = mkDataType "Data.Sequence.Seq" [emptyConstr, consConstr]
 #endif
 
@@ -267,12 +273,12 @@ instance Foldable Digit where
 	foldl f z (Three a b c) = ((z `f` a) `f` b) `f` c
 	foldl f z (Four a b c d) = (((z `f` a) `f` b) `f` c) `f` d
 
-	foldr1 f (One a) = a
+	foldr1 _ (One a) = a
 	foldr1 f (Two a b) = a `f` b
 	foldr1 f (Three a b c) = a `f` (b `f` c)
 	foldr1 f (Four a b c d) = a `f` (b `f` (c `f` d))
 
-	foldl1 f (One a) = a
+	foldl1 _ (One a) = a
 	foldl1 f (Two a b) = a `f` b
 	foldl1 f (Three a b c) = (a `f` b) `f` c
 	foldl1 f (Four a b c d) = ((a `f` b) `f` c) `f` d
@@ -691,13 +697,13 @@ instance Functor ViewL where
 	fmap = fmapDefault
 
 instance Foldable ViewL where
-	foldr f z EmptyL = z
+	foldr _ z EmptyL = z
 	foldr f z (x :< xs) = f x (foldr f z xs)
 
-	foldl f z EmptyL = z
+	foldl _ z EmptyL = z
 	foldl f z (x :< xs) = foldl f (f z x) xs
 
-	foldl1 f EmptyL = error "foldl1: empty view"
+	foldl1 _ EmptyL = error "foldl1: empty view"
 	foldl1 f (x :< xs) = foldl f x xs
 
 instance Traversable ViewL where
@@ -750,13 +756,13 @@ instance Functor ViewR where
 	fmap = fmapDefault
 
 instance Foldable ViewR where
-	foldr f z EmptyR = z
+	foldr _ z EmptyR = z
 	foldr f z (xs :> x) = foldr f (f x z) xs
 
-	foldl f z EmptyR = z
+	foldl _ z EmptyR = z
 	foldl f z (xs :> x) = f (foldl f z xs) x
 
-	foldr1 f EmptyR = error "foldr1: empty view"
+	foldr1 _ EmptyR = error "foldr1: empty view"
 	foldr1 f (xs :> x) = foldr f x xs
 
 instance Traversable ViewR where
@@ -786,7 +792,9 @@ viewRTree (Deep s pr m (Four w x y z)) =
 
 -- Indexing
 
--- | /O(log(min(i,n-i)))/. The element at the specified position
+-- | /O(log(min(i,n-i)))/. The element at the specified position,
+-- which should be a positive integer less than the size of the sequence.
+-- If the position is out of range, 'index' fails with an error.
 index		:: Seq a -> Int -> a
 index (Seq xs) i
   | 0 <= i && i < size xs = case lookupTree i xs of
@@ -848,11 +856,13 @@ lookupDigit i (Four a b c d)
 	sab	= sa + size b
 	sabc	= sab + size c
 
--- | /O(log(min(i,n-i)))/. Replace the element at the specified position
+-- | /O(log(min(i,n-i)))/. Replace the element at the specified position.
+-- If the position is out of range, the original sequence is returned.
 update		:: Int -> a -> Seq a -> Seq a
 update i x	= adjust (const x) i
 
--- | /O(log(min(i,n-i)))/. Update the element at the specified position
+-- | /O(log(min(i,n-i)))/. Update the element at the specified position.
+-- If the position is out of range, the original sequence is returned.
 adjust		:: (a -> a) -> Int -> Seq a -> Seq a
 adjust f i (Seq xs)
   | 0 <= i && i < size xs = Seq (adjustTree (const (fmap f)) i xs)
@@ -911,14 +921,21 @@ adjustDigit f i (Four a b c d)
 -- Splitting
 
 -- | /O(log(min(i,n-i)))/. The first @i@ elements of a sequence.
+-- If @i@ is negative, @'take' i s@ yields the empty sequence.
+-- If the sequence contains fewer than @i@ elements, the whole sequence
+-- is returned.
 take		:: Int -> Seq a -> Seq a
 take i		=  fst . splitAt i
 
 -- | /O(log(min(i,n-i)))/. Elements of a sequence after the first @i@.
+-- If @i@ is negative, @'take' i s@ yields the whole sequence.
+-- If the sequence contains fewer than @i@ elements, the empty sequence
+-- is returned.
 drop		:: Int -> Seq a -> Seq a
 drop i		=  snd . splitAt i
 
 -- | /O(log(min(i,n-i)))/. Split a sequence at a given position.
+-- @'splitAt' i s = ('take' i s, 'drop' i s)@.
 splitAt			:: Int -> Seq a -> (Seq a, Seq a)
 splitAt i (Seq xs)	=  (Seq l, Seq r)
   where	(l, r)		=  split i xs
@@ -958,7 +975,7 @@ splitTree i (Deep _ pr m sf)
 deepL :: Sized a => Maybe (Digit a) -> FingerTree (Node a) -> Digit a -> FingerTree a
 deepL Nothing m sf	= case viewLTree m of
 	Nothing2	-> digitToTree sf
-	Just2 a m'	-> deep (nodeToDigit a) m' sf
+	Just2 a m'	-> Deep (size m + size sf) (nodeToDigit a) m' sf
 deepL (Just pr) m sf	= deep pr m sf
 
 {-# SPECIALIZE deepR :: Digit (Elem a) -> FingerTree (Node (Elem a)) -> Maybe (Digit (Elem a)) -> FingerTree (Elem a) #-}
@@ -966,7 +983,7 @@ deepL (Just pr) m sf	= deep pr m sf
 deepR :: Sized a => Digit a -> FingerTree (Node a) -> Maybe (Digit a) -> FingerTree a
 deepR pr m Nothing	= case viewRTree m of
 	Nothing2	-> digitToTree pr
-	Just2 m' a	-> deep pr m' (nodeToDigit a)
+	Just2 m' a	-> Deep (size pr + size m) pr m' (nodeToDigit a)
 deepR pr m (Just sf)	= deep pr m sf
 
 {-# SPECIALIZE splitNode :: Int -> Node (Elem a) -> Split (Maybe (Digit (Elem a))) (Elem a) #-}
