@@ -1,7 +1,4 @@
-{-# OPTIONS_GHC -fglasgow-exts #-}
-	-- Need GlaExts for the nested forall in defn of Q,
-	-- and the deriving Data, Typeable
-{-# OPTIONS_GHC -w #-}
+{-# OPTIONS_GHC -fno-warn-warnings-deprecations #-}
 -- The above warning supression flag is a temporary kludge.
 -- While working on this module you are encouraged to remove it and fix
 -- any warnings in the module. See
@@ -26,11 +23,11 @@ module Language.Haskell.TH.Syntax(
 
 	Q, runQ, 
 	report,	recover, reify,
-	currentModule, runIO,
+	location, runIO,
 
 	-- Names
 	Name(..), mkName, newName, nameBase, nameModule,
-    showName, showName', NameIs(..),
+        showName, showName', NameIs(..),
 
 	-- The algebraic data types
 	Dec(..), Exp(..), Con(..), Type(..), Cxt, Match(..), 
@@ -38,7 +35,7 @@ module Language.Haskell.TH.Syntax(
 	Lit(..), Pat(..), FieldExp, FieldPat, 
 	Strict(..), Foreign(..), Callconv(..), Safety(..),
 	StrictType, VarStrictType, FunDep(..),
-	Info(..), 
+	Info(..), Loc(..), CharPos,
 	Fixity(..), FixityDirection(..), defaultFixity, maxPrecedence,
 
 	-- Internal functions
@@ -54,8 +51,8 @@ module Language.Haskell.TH.Syntax(
 import Data.PackedString
 import GHC.Base		( Int(..), Int#, (<#), (==#) )
 
-import Data.Generics (Data(..), Typeable, mkConstr, mkDataType)
-import qualified Data.Generics as Generics
+import Data.Data (Data(..), Typeable, mkConstr, mkDataType)
+import qualified Data.Data as Data
 import Data.IORef
 import GHC.IOBase	( unsafePerformIO )
 import Control.Monad (liftM)
@@ -80,7 +77,7 @@ class (Monad m, Functor m) => Quasi m where
  
 	-- Inspect the type-checker's environment
   qReify :: Name -> m Info
-  qCurrentModule :: m String
+  qLocation :: m Loc
 
 	-- Input/output (dangerous)
   qRunIO :: IO a -> m a
@@ -105,9 +102,9 @@ instance Quasi IO where
   qReport True  msg = hPutStrLn stderr ("Template Haskell error: " ++ msg)
   qReport False msg = hPutStrLn stderr ("Template Haskell error: " ++ msg)
 
-  qReify v       = badIO "reify"
-  qCurrentModule = badIO "currentModule"
-  qRecover a b   = badIO "recover"	-- Maybe we could fix this?
+  qReify _     = badIO "reify"
+  qLocation    = badIO "currentLocation"
+  qRecover _ _ = badIO "recover" -- Maybe we could fix this?
 
   qRunIO m = m
   
@@ -156,10 +153,10 @@ recover (Q r) (Q m) = Q (qRecover r m)
 reify :: Name -> Q Info
 reify v = Q (qReify v)
 
--- | 'currentModule' gives you the name of the module in which this 
+-- | 'location' gives you the 'Location' at which this
 -- computation is spliced.
-currentModule :: Q String
-currentModule = Q qCurrentModule
+location :: Q Loc
+location = Q qLocation
 
 -- |The 'runIO' function lets you run an I\/O computation in the 'Q' monad.
 -- Take care: you are guaranteed the ordering of calls to 'runIO' within 
@@ -172,12 +169,12 @@ runIO :: IO a -> Q a
 runIO m = Q (qRunIO m)
 
 instance Quasi Q where
-  qNewName        = newName
-  qReport 	 = report
-  qRecover  	 = recover 
-  qReify    	 = reify
-  qCurrentModule = currentModule
-  qRunIO         = runIO
+  qNewName  = newName
+  qReport   = report
+  qRecover  = recover 
+  qReify    = reify
+  qLocation = location
+  qRunIO    = runIO
 
 
 ----------------------------------------------------
@@ -264,8 +261,8 @@ instance (Lift a, Lift b, Lift c, Lift d, Lift e, Lift f, Lift g)
 
 
 trueName, falseName :: Name
-trueName  = mkNameG DataName "base" "GHC.Base" "True"
-falseName = mkNameG DataName "base" "GHC.Base" "False"
+trueName  = mkNameG DataName "ghc-prim" "GHC.Bool" "True"
+falseName = mkNameG DataName "ghc-prim" "GHC.Bool" "False"
 
 nothingName, justName :: Name
 nothingName = mkNameG DataName "base" "Data.Maybe" "Nothing"
@@ -357,11 +354,14 @@ instance Data NameFlavour where
      toConstr (NameG _ _ _) = con_NameG
      dataTypeOf _ = ty_NameFlavour
 
-con_NameS = mkConstr ty_NameFlavour "NameS" [] Generics.Prefix
-con_NameQ = mkConstr ty_NameFlavour "NameQ" [] Generics.Prefix
-con_NameU = mkConstr ty_NameFlavour "NameU" [] Generics.Prefix
-con_NameL = mkConstr ty_NameFlavour "NameL" [] Generics.Prefix
-con_NameG = mkConstr ty_NameFlavour "NameG" [] Generics.Prefix
+con_NameS, con_NameQ, con_NameU, con_NameL, con_NameG :: Data.Constr
+con_NameS = mkConstr ty_NameFlavour "NameS" [] Data.Prefix
+con_NameQ = mkConstr ty_NameFlavour "NameQ" [] Data.Prefix
+con_NameU = mkConstr ty_NameFlavour "NameU" [] Data.Prefix
+con_NameL = mkConstr ty_NameFlavour "NameL" [] Data.Prefix
+con_NameG = mkConstr ty_NameFlavour "NameG" [] Data.Prefix
+
+ty_NameFlavour :: Data.DataType
 ty_NameFlavour = mkDataType "Language.Haskell.TH.Syntax.NameFlavour"
                             [con_NameS, con_NameQ, con_NameU,
                              con_NameL, con_NameG]
@@ -378,9 +378,9 @@ nameBase :: Name -> String
 nameBase (Name occ _) = occString occ
 
 nameModule :: Name -> Maybe String
-nameModule (Name _ (NameQ m))   = Just (modString m)
+nameModule (Name _ (NameQ m))     = Just (modString m)
 nameModule (Name _ (NameG _ _ m)) = Just (modString m)
-nameModule other_name		= Nothing
+nameModule _                      = Nothing
 
 mkName :: String -> Name
 -- The string can have a '.', thus "Foo.baz",
@@ -414,8 +414,8 @@ mkNameL :: String -> Uniq -> Name	-- Only used internally
 mkNameL s (I# u) = Name (mkOccName s) (NameL u)
 
 mkNameG :: NameSpace -> String -> String -> String -> Name	-- Used for 'x etc, but not available
-mkNameG ns pkg mod occ 					-- to the programmer
-  = Name (mkOccName occ) (NameG ns (mkPkgName pkg) (mkModName mod))
+mkNameG ns pkg modu occ 					-- to the programmer
+  = Name (mkOccName occ) (NameG ns (mkPkgName pkg) (mkModName modu))
 
 mkNameG_v, mkNameG_tc, mkNameG_d :: String -> String -> String -> Name
 mkNameG_v  = mkNameG VarName
@@ -435,18 +435,18 @@ instance Eq NameFlavour where
 instance Ord NameFlavour where
 	-- NameS < NameQ < NameU < NameL < NameG
   NameS `compare` NameS = EQ
-  NameS `compare` other = LT
+  NameS `compare` _     = LT
 
   (NameQ _)  `compare` NameS      = GT
   (NameQ m1) `compare` (NameQ m2) = m1 `compare` m2
-  (NameQ _)  `compare` other      = LT
+  (NameQ _)  `compare` _          = LT
 
   (NameU _)  `compare` NameS      = GT
   (NameU _)  `compare` (NameQ _)  = GT
   (NameU u1) `compare` (NameU u2) | u1  <# u2 = LT
 				  | u1 ==# u2 = EQ
 				  | otherwise = GT
-  (NameU _)  `compare` other = LT
+  (NameU _)  `compare` _     = LT
 
   (NameL _)  `compare` NameS      = GT
   (NameL _)  `compare` (NameQ _)  = GT
@@ -454,12 +454,12 @@ instance Ord NameFlavour where
   (NameL u1) `compare` (NameL u2) | u1  <# u2 = LT
 				  | u1 ==# u2 = EQ
 				  | otherwise = GT
-  (NameL _)  `compare` other      = LT
+  (NameL _)  `compare` _          = LT
 
   (NameG ns1 p1 m1) `compare` (NameG ns2 p2 m2) = (ns1 `compare` ns2) `thenCmp`
                                             (p1 `compare` p2) `thenCmp`
 					    (m1 `compare` m2) 
-  (NameG _ _ _)    `compare` other	  = GT
+  (NameG _ _ _)    `compare` _ = GT
 
 data NameIs = Alone | Applied | Infix
 
@@ -484,11 +484,11 @@ showName' ni nm
 	-- We may well want to distinguish them in the end.
 	-- Ditto NameU and NameL
         nms = case nm of
-                    Name occ NameS          -> occString occ
-                    Name occ (NameQ m)      -> modString m ++ "." ++ occString occ
-                    Name occ (NameG ns p m) -> modString m ++ "." ++ occString occ
-                    Name occ (NameU u)      -> occString occ ++ "_" ++ show (I# u)
-                    Name occ (NameL u)      -> occString occ ++ "_" ++ show (I# u)
+                    Name occ NameS         -> occString occ
+                    Name occ (NameQ m)     -> modString m ++ "." ++ occString occ
+                    Name occ (NameG _ _ m) -> modString m ++ "." ++ occString occ
+                    Name occ (NameU u)     -> occString occ ++ "_" ++ show (I# u)
+                    Name occ (NameL u)     -> occString occ ++ "_" ++ show (I# u)
 
         pnam = classify nms
 
@@ -516,14 +516,29 @@ tupleTypeName 0 = mk_tup_name 0 TcClsName
 tupleTypeName 1 = error "tupleTypeName 1"
 tupleTypeName n = mk_tup_name (n-1) TcClsName 
 
+mk_tup_name :: Int -> NameSpace -> Name
 mk_tup_name n_commas space
-  = Name occ (NameG space (mkPkgName "base") tup_mod)
+  = Name occ (NameG space (mkPkgName "ghc-prim") tup_mod)
   where
     occ = mkOccName ('(' : replicate n_commas ',' ++ ")")
-    tup_mod = mkModName "Data.Tuple"
+    -- XXX Should it be GHC.Unit for 0 commas?
+    tup_mod = mkModName "GHC.Tuple"
 
 
 
+-----------------------------------------------------
+--		Locations
+-----------------------------------------------------
+
+data Loc
+  = Loc { loc_filename :: String
+	, loc_package  :: String
+	, loc_module   :: String
+	, loc_start    :: CharPos
+	, loc_end      :: CharPos }
+
+type CharPos = (Int, Int)	-- Line and character position
+
 
 -----------------------------------------------------
 --
@@ -592,6 +607,7 @@ data Lit = CharL Char
                                 -- the moment. Maybe that doesn't matter?
          | RationalL Rational   -- Ditto
          | IntPrimL Integer
+         | WordPrimL Integer
          | FloatPrimL Rational
          | DoublePrimL Rational
     deriving( Show, Eq, Data, Typeable )
@@ -741,5 +757,5 @@ cmpEq _  = False
 
 thenCmp :: Ordering -> Ordering -> Ordering
 thenCmp EQ o2 = o2
-thenCmp o1 o2 = o1
+thenCmp o1 _  = o1
 
