@@ -1,5 +1,6 @@
 /* -----------------------------------------------------------------------------
  *
+ * (c) Lennart Augustsson
  * (c) The GHC Team, 1998-2000
  *
  * Miscellaneous support for floating-point primitives
@@ -80,6 +81,44 @@ __encodeDouble (I_ size, StgByteArray ba, I_ e) /* result = s * 2^e */
     return r;
 }
 
+StgDouble
+__2Int_encodeDouble (I_ j_high, I_ j_low, I_ e)
+{
+  StgDouble r;
+  
+  /* assuming 32 bit ints */
+  ASSERT(sizeof(int          ) == 4            );
+
+  r = (StgDouble)((unsigned int)j_high);
+  r *= 4294967296.0; /* exp2f(32); */
+  r += (StgDouble)((unsigned int)j_low);
+  
+  /* Now raise to the exponent */
+  if ( r != 0.0 ) /* Lennart suggests this avoids a bug in MIPS's ldexp */
+    r = ldexp(r, e);
+  
+  /* sign is encoded in the size */
+  if (j_high < 0)
+    r = -r;
+  
+  return r;
+}
+
+/* Special version for words */
+StgDouble
+__word_encodeDouble (W_ j, I_ e)
+{
+  StgDouble r;
+  
+  r = (StgDouble)j;
+  
+  /* Now raise to the exponent */
+  if ( r != 0.0 ) /* Lennart suggests this avoids a bug in MIPS's ldexp */
+    r = ldexp(r, e);
+  
+  return r;
+}
+
 /* Special version for small Integers */
 StgDouble
 __int_encodeDouble (I_ j, I_ e)
@@ -140,6 +179,21 @@ __int_encodeFloat (I_ j, I_ e)
   return r;
 }
 
+/* Special version for small positive Integers */
+StgFloat
+__word_encodeFloat (W_ j, I_ e)
+{
+  StgFloat r;
+  
+  r = (StgFloat)j;
+  
+  /* Now raise to the exponent */
+  if ( r != 0.0 ) /* Lennart suggests this avoids a bug in MIPS's ldexp */
+    r = ldexp(r, e);
+  
+  return r;
+}
+
 /* This only supports IEEE floating point */
 
 void
@@ -202,6 +256,51 @@ __decodeDouble (MP_INT *man, I_ *exp, StgDouble dbl)
     }
 }
 
+void
+__decodeDouble_2Int (I_ *man_sign, W_ *man_high, W_ *man_low, I_ *exp, StgDouble dbl)
+{
+    /* Do some bit fiddling on IEEE */
+    unsigned int low, high; 	     	/* assuming 32 bit ints */
+    int sign, iexp;
+    union { double d; unsigned int i[2]; } u;	/* assuming 32 bit ints, 64 bit double */
+
+    ASSERT(sizeof(unsigned int ) == 4            );
+    ASSERT(sizeof(dbl          ) == 8            );
+    ASSERT(sizeof(dbl          ) == SIZEOF_DOUBLE);
+
+    u.d = dbl;	    /* grab chunks of the double */
+    low = u.i[L];
+    high = u.i[H];
+
+    if (low == 0 && (high & ~DMSBIT) == 0) {
+	*man_low = 0;
+	*man_high = 0;
+	*exp = 0L;
+    } else {
+	iexp = ((high >> 20) & 0x7ff) + MY_DMINEXP;
+	sign = high;
+
+	high &= DHIGHBIT-1;
+	if (iexp != MY_DMINEXP)	/* don't add hidden bit to denorms */
+	    high |= DHIGHBIT;
+	else {
+	    iexp++;
+	    /* A denorm, normalize the mantissa */
+	    while (! (high & DHIGHBIT)) {
+		high <<= 1;
+		if (low & DMSBIT)
+		    high++;
+		low <<= 1;
+		iexp--;
+	    }
+	}
+        *exp = (I_) iexp;
+	*man_low = low;
+	*man_high = high;
+	*man_sign = (sign < 0) ? -1 : 1;
+    }
+}
+
 void
 __decodeFloat (MP_INT *man, I_ *exp, StgFloat flt)
 {
@@ -255,6 +354,44 @@ __decodeFloat (MP_INT *man, I_ *exp, StgFloat flt)
    based on defs in GNU libc <ieee754.h>
 */
 
+void
+__decodeFloat_Int (I_ *man, I_ *exp, StgFloat flt)
+{
+    /* Do some bit fiddling on IEEE */
+    int high, sign; 	    	    /* assuming 32 bit ints */
+    union { float f; int i; } u;    /* assuming 32 bit float and int */
+
+    ASSERT(sizeof(int          ) == 4            );
+    ASSERT(sizeof(flt          ) == 4            );
+    ASSERT(sizeof(flt          ) == SIZEOF_FLOAT );
+
+    u.f = flt;	    /* grab the float */
+    high = u.i;
+
+    if ((high & ~FMSBIT) == 0) {
+	*man = 0;
+	*exp = 0;
+    } else {
+	*exp = ((high >> 23) & 0xff) + MY_FMINEXP;
+	sign = high;
+
+	high &= FHIGHBIT-1;
+	if (*exp != MY_FMINEXP)	/* don't add hidden bit to denorms */
+	    high |= FHIGHBIT;
+	else {
+	    (*exp)++;
+	    /* A denorm, normalize the mantissa */
+	    while (! (high & FHIGHBIT)) {
+		high <<= 1;
+		(*exp)--;
+	    }
+	}
+	*man = high;
+	if (sign < 0)
+	    *man = - *man;
+    }
+}
+
 union stg_ieee754_flt
 {
    float f;
