@@ -1,6 +1,6 @@
 /* -----------------------------------------------------------------------------
  *
- * (c) The GHC Team 1998-2006
+ * (c) The GHC Team 1998-2008
  *
  * Generational garbage collector: evacuation functions
  *
@@ -27,6 +27,7 @@
 #define REGPARM1
 #endif
 
-REGPARM1 StgClosure * evacuate (StgClosure *q);
+REGPARM1 void evacuate  (StgClosure **p);
+REGPARM1 void evacuate1 (StgClosure **p);
 
 extern lnat thunk_selector_depth;
