@@ -1,6 +1,6 @@
 /* -----------------------------------------------------------------------------
  *
- * (c) The GHC Team 1998-2006
+ * (c) The GHC Team 1998-2008
  *
  * Generational garbage collector: scavenging functions
  *
@@ -11,8 +11,10 @@
  *
  * ---------------------------------------------------------------------------*/
 
-void scavenge                ( step * );
-void scavenge_mark_stack     ( void );
-void scavenge_large          ( step * );
-void scavenge_static         ( void );
-void scavenge_mutable_list   ( generation *g );
+void    scavenge_loop (void);
+void    scavenge_mutable_list (generation *g);
+
+#ifdef THREADED_RTS
+void    scavenge_loop1 (void);
+void    scavenge_mutable_list1 (generation *g);
+#endif
